//************************************************************************
// Liest eine Intel8,32,MCS HEX-Brenndatei
// Epromer V8.xx
//
// Holger Klabunde
// 30.10.2002
// Compiler Visual C++ V5.0
//************************************************************************
#include "stdafx.h"

#include "Global.h"
#include "protos.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Liest eine Intel '.hex' ASCII-Datei mit 32Bit Adressierung
// Aufbau der '.hex' Zeile mit TT="02"
// :CCXXXXTTAAAAPPLL
// CC		Anzahl der Datenbytes dieser Zeile
// XXXX	Nicht benutzt
// TT		02=> Zeile enthlt neuen Offset
// AAAA	16 Bit zustzlicher Adressanteil ber A3
//      A4 bis A7 dieses Anteils sind IMMER 0. Genauer gesagt
//      sind hier A8 bis A19 enthalten. 
//
// Aufbau der '.hex' Zeile mit TT="04"
// :CCXXXXTTAAAAPPLL
// CC		Anzahl der Datenbytes dieser Zeile
// XXXX	Nicht benutzt
// TT		04=> Adresse A16 bis A31
// AAAA	16 Bit zustzlicher Adressanteil ber A15
//
// Aufbau der '.hex' Zeile mit TT="00" oder "01"
// :CCAAAATTDDDDDDDDDDDDPPLL
// CC		Anzahl der Datenbytes dieser Zeile
// AAAA	16 Bit Offset zur Grundadresse des ersten Datenbytes dieser Zeile
// TT		00=> Datenzeile		01=> Letzte Zeile, Kennzeichen Datenende
// DD		Datenbytes
// PP 	Prfsumme: Lowbyte des Zweierkomplements ber die Summe aller Bytes,
//    	nicht nur der Datenbytes
// LL		Zeilenende (CR,LF)
// Ende der Brenndatei bei Zeile ':00000001FF'
int CAVRPromDlg::ReadIntelHex()
//************************************************
{
 char *n;
 unsigned char tst,check,check1,tmp; // Zeile gltig ?
 unsigned long i,checksum,byl,adr;

 checksum=0;

 n=fgets(line,ARRAYSIZE,in);
 if(n!=NULL)
  {
	linecount++; //Nummer der gelesenen Zeile
	if(strncmp(&line[0],":",1)!=0) //Anfang der Zeile vorhanden ?
	 { NoLineBegin(); return NO_INPUT; }

	byl=Asc2Byte(&line[1]); // Lnge der Hex-Zeile
	checksum+=byl; 			// gehrt zur Checksumme
	tst=Asc2Byte(&line[7]); //Gltige Zeile ?
	checksum+=tst;          // gehrt zur Checksumme

//Chiplab macht kein 0D0A ans Ende der letzten Zeile. Deswegen teste
//ich hier die Zeilenlnge nicht, sondern gehe gleich raus.
	if((tst==1)&&(byl==0)) { return END_HEX; }//Ende der Datei gefunden

    ClearHexString(line,ARRAYSIZE); //berraschungen wegfiltern

	if(strlen(line)!=(byl*2+11)) //Lnge der Zeile berechnen
	 { WrongHexLineLen(strlen(line),byl*2+11); return NO_INPUT; }


	adr=(unsigned long)Asc2Word(&line[3],4); // Brennadresse

	checksum+=adr & 0xFF; // Low und Highbyte zur Checksumme addieren
	checksum+=(adr>>8) & 0xFF;

	if((byl!=0)&&(tst!=1)) // Bytes vorhanden und Zeile gltig ?
	 {

//Hier kommen zwei Offsets. Die beiden schlieen sich eigentlich aus weil
//sie zum Teil im selben Adressbereich liegen. Das auftauchen eines neuen
//Offsets zerstrt deshalb den Vorgnger. Anders macht das keinen Sinn.

	  if(tst==2) //Adressteil A4-A19
		{
		 offset=(unsigned long)Asc2Word(&line[9],4);
		 checksum+=offset & 0xFF; // Low und Highbyte zur Checksumme addieren
		 checksum+=(offset>>8) & 0xFF;
		 offset=(offset<<4); //Auf richtige Stelle schieben
		}

	  if(tst==4) //Adressteil A16-A31
		{
		 offset=(unsigned long)Asc2Word(&line[9],4);
		 checksum+=offset & 0xFF; // Low und Highbyte zur Checksumme addieren
		 checksum+=(offset>>8) & 0xFF;
		 offset=(offset<<16); //Auf richtige Stelle schieben
		}

	  adr+=offset; //Wahre Adresse berechnen

	  if(tst==0) //Es folgen Datenbytes
		{
		 for(i=0; i<(byl*2); i+=2) // Alle Bytes der Zeile lesen
		  {
			tmp=Asc2Byte(&line[9+i]); //Byte aus der Zeile lesen
			if(adr<BLOCKSIZE)
			 {
			  if(data[adr]!=0xFF) { BadHex(); return NO_INPUT; } //Zelle doppelt belegt worden !
			  data[adr]=tmp;
             }
			checksum+=tmp; // Checksumme updaten
            adr++;
		  }

         //Alle Datenbytes gelesen, dann bei adr eins abziehen
		 //Das letzte adr++ war nmlich eins zuviel
         adr--;

		}//if tst==0

      highadr=__max(highadr,adr); //Neue Adresse hher ?

	  // Checksumme= Lowbyte vom Zweierkomplement der Summe aller Bytes
	  //				 der Zeile (nicht nur der Datenbytes)
	  check=(unsigned char)(((~checksum)+1) & 0xFF);
	  check1=Asc2Byte(&line[9+byl*2]); //2 Byte Prfsumme hinter den Datenbytes

	  if(testcheck>0)
	   {
	    if(check!=check1) { BadChecksum(check1,check); return NO_INPUT; }
	   }

	  if(tst==2) return HEADER_ONLY; //Nur neue untere Offsetadresse gelesen
	  if(tst==4) return HEADER_ONLY; //Nur neue obere Offsetadresse gelesen

	 }
  }

 if(highadr==0) return NO_INPUT;
 return NO_ERRORS;
}

