//************************************************************************
// Liest eine Atmel Assembler ROM HEX-Brenndatei
// AVRPROM V1.xx
//
// Holger Klabunde
// 06.09.2001
// Compiler Visual C++ V5.0
//*****************************************************
#include "stdafx.h"

#include "Global.h"
#include "protos.h"
#include "port_io.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Liest eine AtmelAssembler-ROM '.rom' oder '.eep' ASCII Zeile
// Aufbau einer Zeile
// AAAA(AA):DD(DD)LL
// AAAA(AA)	zwei oder 3 Byte Adresse
//          bei zwei Byte Adresse EEPROM Daten -> DD = 1 Byte Daten
//			bei drei Byte Adresse Programdaten -> DDDD 2 Byte Befehl
// DD(DD)	1 oder 2 Datenbytes
// LL			Zeilenende (CR,LF)
int CAVRPromDlg::ReadROMHex(int mode)
//************************************************
{
 char *n;
 unsigned long adr;     //Adresse
 unsigned long dataword; //8 oder 16Bit Datenwort


 n=fgets(line,ARRAYSIZE,in);
 if(n!=NULL)
  {
   linecount++; //Nummer der gelesenen Zeile

//Fehlererkennung
//1. Mal sehen ob das ':' da ist.
//Weitere Fehlererkennungsmglichkeiten
//2. Zeilenlnge bei AAAA(AA) mu stimmen
//3. Alle Zeichen auer ':' mssen zwischen 0-F sein

//Bytedaten lesen
   if(mode==8) //Zeile soll Bytedaten enthalten
	{ 
     if(line[4]!=':') //Sehen ob das ':' da ist
	  {
       Msg("Das ':' fehlt in der Zeile");
	   return NO_INPUT;
	  }

     ClearHexString(line,ARRAYSIZE); //berraschungen wegfiltern

	 if(strlen(line)!=7) //Lnge der Zeile berechnen
	  {
	   WrongHexLineLen(strlen(line),7);
	   return NO_INPUT;
	  }

	 adr=(unsigned long)Asc2Word(&line[0],4); // 16 Bit Brennadresse
     dataword=(unsigned long) Asc2Byte(&line[5]); //Das einzige Byte der Zeile lesen

	 if(adr<BLOCKSIZE )
	  {
	   if(data[adr]!=0xFF) { BadHex(); return NO_INPUT; } //Zelle doppelt belegt worden !
	   data[adr]=(unsigned char)(dataword&0xFF); //Byte in Brennarray eintragen
      }
//     else return NO_INPUT;

     highadr=__max(highadr,adr); //Hchste Adresse ermitteln

    }//if mode==8

//Worddaten lesen
   if(mode==16) //Zeile soll Bytedaten enthalten
    { 
     if(line[6]!=':') //Sehen ob das ':' da ist
      {
       Msg("Das ':' fehlt in der Zeile");
	   return NO_INPUT;
	  }

     ClearHexString(line,ARRAYSIZE); //berraschungen wegfiltern

	 if(strlen(line)!=11) //Lnge der Zeile berechnen
	  {
	   WrongHexLineLen(strlen(line),11);
	   return NO_INPUT;
	  }

	 adr=(unsigned long)Asc2Word(&line[0],6); // 24 Bit Brennadresse
	 highadr=__max(highadr,(adr*2)+1); //Hchste Byte-Adresse ermitteln

     dataword=(unsigned long) Asc2Word(&line[7],4); //Das einzige Word der Zeile lesen

     adr<<=2; //UInt Daten in Byte-Array !!!

	 if(adr<BLOCKSIZE )
	  {
	   if(data[adr]!=0xFF) { BadHex(); return NO_INPUT; } //Zelle doppelt belegt worden !
	   data[adr]=(unsigned char)(dataword&0xFF);      //LowByte
	   if(data[adr+1]!=0xFF) { BadHex(); return NO_INPUT; } //Zelle doppelt belegt worden !
	   data[adr+1]=(unsigned char)((dataword>>8)&0xFF); //HighByte
      }

    }//if mode==16


  } //if n!=NULL

 if(n==NULL) return END_HEX;
 return NO_ERRORS;
}

