//************************************************************************
// Liest eine Motorola 32Bit S3-Record HEX-Brenndatei
// mit 32Bit Adressierung und Byte Daten
// S1,S2 sind auch enthalten !!
// AVRPrommer V1.4x
//
// Holger Klabunde
// 06.09.2001
// Compiler Visual C++ V5.0
//************************************************************************
#include "stdafx.h"
#include <stdio.h>

#include "Global.h"
#include "protos.h"
#include "port_io.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Liest eine Motorola S123 '.hex' ASCII Zeile
// Aufbau einer '.hex' Zeile
// S1CCAAAADDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDPPLL
// S2CCAAAAAADDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDPPLL
// S3CCAAAAAAAADDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDPPLL
// CC			Anzahl aller Bytes dieser Zeile inklusive Checksumme
// AAAAAAAA	16,24,32 Bit Adresse des ersten Datenbytes dieser Zeile
// DD			Datenbytes
// PP 	Prfsumme: Lowbyte des Einerkomplements ber die Summe aller Bytes,
//    	nicht nur der Datenbytes auer dem 'S1,2,3'
// LL		Zeilenende (CR,LF)
// Ende der Brenndatei bei Zeilen
// S1 => "S9030000FC"
// S2 => "S804000000FB"
// S3 => "S70500000000FA"

int CAVRPromDlg::ReadS123Hex()
//************************************************
{
 char *n;
 unsigned char check,check1,tmp; // Zeile gltig ?
 unsigned long i,checksum,byl,databytes,adr;

 checksum=0;

 n=fgets(line,ARRAYSIZE,in);
 if(n!=NULL)
  {
    linecount++; //Nummer der gelesenen Zeile

	if(strncmp(&line[0],"S0",2)==0) //Header Record nur berlesen
	 {
	  return HEADER_ONLY;
	 }

	if(strncmp(&line[0],"S9",2)==0) //Ende der Hex-Datei
	 { return END_HEX; }

	if(strncmp(&line[0],"S8",2)==0) //Ende der Hex-Datei
	 { return END_HEX; }

	if(strncmp(&line[0],"S7",2)==0) //Ende der Hex-Datei
	 { return END_HEX; }

//S1 Records sind auch in S3 Hex-Dateien enthalten
	if((strncmp(&line[0],"S1",2)!=0)&&
		(strncmp(&line[0],"S2",2)!=0)&&
		(strncmp(&line[0],"S3",2)!=0)	  ) //Anfang der Zeile vorhanden ?
	 { NoLineBegin(); return NO_INPUT; }

	if(strncmp(&line[0],"S1",2)==0) //S1 lesen
	 {
	  byl=Asc2Byte(&line[2]); // Lnge der Hex-Zeile ohne 'S1'
	  checksum+=byl; 			// gehrt zur Checksumme

	  if(byl==3) return HEADER_ONLY; //Keine Daten in dieser Zeile

      ClearHexString(line,ARRAYSIZE); //berraschungen wegfiltern

	  if(strlen(line)!=(byl*2+4)) //Lnge der Zeile berechnen
		{ WrongHexLineLen(strlen(line),byl*2+4); return NO_INPUT; }

	  adr=(unsigned long)Asc2Word(&line[4],4); // 16 Bit Brennadresse

	  checksum+=adr & 0xFF; // Low und Highbyte zur Checksumme addieren
	  checksum+=(adr>>8) & 0xFF;

	  databytes=byl-3; //16 Bit Adresse und Checksumme abziehen

	  if(databytes>0) // Bytes vorhanden ?
		{
		 for(i=0; i<(databytes*2); i+=2) // Alle Bytes der Zeile lesen
		  {
			tmp=Asc2Byte(&line[8+i]);
			if(adr<BLOCKSIZE)
			 {
 	  	      if(data[adr]!=0xFF) { BadHex(); return NO_INPUT; } //Zelle doppelt belegt worden !
			  data[adr]=tmp;
			 }
			checksum+=tmp; // Checksumme updaten
			adr++;
		  }

         //Alle Datenbytes gelesen, dann bei adr eins abziehen
		 //Das letzte adr++ war dann eins zuviel
         adr--;

         highadr=__max(highadr,adr); //Neue Adresse hher ?

		 // Checksumme= Lowbyte vom Einerkomplement der Summe aller Bytes
		 //				 der Zeile auer dem 'S1'
		 check=(unsigned char)((~checksum) & 0xFF);
		 check1=Asc2Byte(&line[2+byl*2]); //2 Byte Prfsumme nach den Datenbytes

         if(testcheck>0)
		  {
	       if(check!=check1)	{ BadChecksum(check1,check); return NO_INPUT; }
		  }

		} //if byl!=0
	 }//if S1
//Ende S1 Records

//S2 Records sind auch in S3 Dateien enthalten
	if(strncmp(&line[0],"S2",2)==0) //S2 lesen
	 {
	  byl=Asc2Byte(&line[2]); // Lnge der Hex-Zeile ohne 'S2'
	  checksum+=byl; 			// gehrt zur Checksumme

	  if(byl==4) return HEADER_ONLY; //Keine Daten in dieser Zeile

      ClearHexString(line,ARRAYSIZE); //berraschungen wegfiltern

	  if(strlen(line)!=(byl*2+4)) //Lnge der Zeile berechnen
		{ WrongHexLineLen(strlen(line),byl*2+4); return NO_INPUT; }

	  adr=(unsigned long)Asc2Word(&line[4],6); // 24 Bit Brennadresse

	  checksum+=adr & 0xFF; // Low,Mid, Highbyte zur Checksumme addieren
	  checksum+=(adr>>8) & 0xFF;
	  checksum+=(adr>>16) & 0xFF;

	  databytes=byl-4; //24 Bit Adresse und Checksumme abziehen

	  if(databytes>0) // Bytes vorhanden ?
		{
		 for(i=0; i<(databytes*2); i+=2) // Alle Bytes der Zeile lesen
		  {
			tmp=Asc2Byte(&line[10+i]);
			if(adr<BLOCKSIZE)
			 {
 	  	      if(data[adr]!=0xFF) { BadHex(); return NO_INPUT; } //Zelle doppelt belegt worden !
              data[adr]=tmp;
             }

			checksum+=tmp; // Checksumme updaten
			adr++;
		  }

         //Alle Datenbytes gelesen, dann bei adr eins abziehen
		 //Das letzte adr++ war dann eins zuviel
         adr--;

         highadr=__max(highadr,adr); //Neue Adresse hher ?

		 // Checksumme= Lowbyte vom Einerkomplement der Summe aller Bytes
		 //				 der Zeile auer dem 'S2'
		 check=(unsigned char)((~checksum) & 0xFF);
		 check1=Asc2Byte(&line[2+byl*2]); //2 Byte Prfsumme nach den Datenbytes

         if(testcheck>0)
		  {
	       if(check!=check1)	{ BadChecksum(check1,check); return NO_INPUT; }
		  }

		} //if byl!=0
	 }//if S2
//Ende S2 Records

//S3 Records lesen
	if(strncmp(&line[0],"S3",2)==0) //S3 lesen
	 {
	  byl=Asc2Byte(&line[2]); // Lnge der Hex-Zeile ohne 'S3'
	  checksum+=byl; 			// gehrt zur Checksumme

	  if(byl==5) return HEADER_ONLY; //Keine Daten in dieser Zeile

      ClearHexString(line,ARRAYSIZE); //berraschungen wegfiltern

	  if(strlen(line)!=(byl*2+4)) //Lnge der Zeile berechnen
		{ WrongHexLineLen(strlen(line),byl*2+4); return NO_INPUT; }

	  adr=(unsigned long)Asc2Word(&line[4],8); // 32 Bit Brennadresse

	  checksum+=adr & 0xFF; // Low,Mid, Highbyte zur Checksumme addieren
	  checksum+=(adr>>8) & 0xFF;
	  checksum+=(adr>>16) & 0xFF;
	  checksum+=(adr>>24) & 0xFF;

	  databytes=byl-5; //32 Bit Adresse und Checksumme abziehen

	  if(databytes>0) // Bytes vorhanden ?
		{
		 for(i=0; i<(databytes*2); i+=2) // Alle Bytes der Zeile lesen
		  {
			tmp=Asc2Byte(&line[12+i]);
			if(adr<BLOCKSIZE)
			 {
 	  	      if(data[adr]!=0xFF) { BadHex(); return NO_INPUT; } //Zelle doppelt belegt worden !
               data[adr]=tmp;
             }

			checksum+=tmp; // Checksumme updaten
			adr++;
		  }

         //Alle Datenbytes gelesen, dann bei adr eins abziehen
		 //Das letzte adr++ war dann eins zuviel
         adr--;

         highadr=__max(highadr,adr); //Neue Adresse hher ?

		 // Checksumme= Lowbyte vom Einerkomplement der Summe aller Bytes
		 //				 der Zeile auer dem 'S3'
		 check=(unsigned char)((~checksum) & 0xFF);
		 check1=Asc2Byte(&line[2+byl*2]); //2 Byte Prfsumme nach den Datenbytes

         if(testcheck>0)
		  {
	       if(check!=check1)	{ BadChecksum(check1,check); return NO_INPUT; }
		  }

		} //if byl!=0
	 }//if S3
//Ende S3 Records

  } //if n!=NULL

 if(highadr==0) return NO_INPUT;
 return NO_ERRORS;
}

